#include "camera.h"

Camera::Camera()
{
    SetLookAt(QVector3D(0.0, 0.0, 10.0),
              QVector3D(0.0, 0.0, 0.0),
              QVector3D(0.0, 1.0, 0.0));

    scale = 1.0;
    nearPlane = -1.0;
    farPlane = 1.0;
    orthoA = 0.0;
    orthoB = -1.0;
}

Camera::Camera(QVector3D eye, QVector3D at, QVector3D up)
{
    SetLookAt(eye, at, up);
}

void Camera::SetPosition(QVector3D position)
{
    this->position = position;
}

void Camera::SetLookAt(QVector3D eye, QVector3D at, QVector3D up)
{
    position = eye;
    view = at - eye;
    this->up = up;

    side = QVector3D::crossProduct(view, up);
    this->up = QVector3D::crossProduct(side, view);

    view.normalize();
    this->up.normalize();
    side.normalize();
}

void Camera::SetScale(qreal scale)
{
    this->scale = scale;
}

QMatrix4x4 Camera::ProjectionMatrix(int viewportWidth, int viewportHeight,
                                    qreal nearPlane, qreal farPlane)
{
    QMatrix4x4 projectionMatrix;

    qreal aspect = static_cast<qreal>(viewportWidth) /
                   static_cast<qreal>(viewportHeight);

    projectionMatrix.setToIdentity();
    projectionMatrix.ortho(-1.0*aspect, 1.0*aspect, -1.0, 1.0,
                           nearPlane, farPlane);

    this->nearPlane = nearPlane;
    this->farPlane = farPlane;
    orthoA = (farPlane+nearPlane)/(farPlane-nearPlane);
    orthoB = -2.0/(farPlane-nearPlane);  
    this->width = viewportWidth;
    this->height = viewportHeight;

    return projectionMatrix;
}

QMatrix4x4 Camera::ModelViewMatrix()
{
    QMatrix4x4 modelViewMatrix;

    modelViewMatrix.setToIdentity();
    modelViewMatrix.lookAt(position, position + view, up);

    return modelViewMatrix;
}
