#ifndef CAMERA_H
#define CAMERA_H

#include <QMatrix4x4>
#include <QVector3D>

class Camera
{
public:
    Camera();
    Camera(QVector3D eye, QVector3D at, QVector3D up);

    QVector3D Position() const { return position; }
    qreal Scale() const { return scale; }
    qreal NearPlane() const { return nearPlane; }
    qreal FarPlane() const { return farPlane; }
    qreal OrthoA() const { return orthoA; }
    qreal OrthoB() const { return orthoB; }
    int viewportWidth() const { return width; }
    int viewportHeight() const { return height; }

    void SetPosition(QVector3D position);
    void SetLookAt(QVector3D eye, QVector3D at, QVector3D up);
    void SetScale(qreal scale);

    QMatrix4x4 ProjectionMatrix(int viewportWidth, int viewportHeight,
                                qreal nearPlane, qreal farPlane);
    QMatrix4x4 ModelViewMatrix();

private:
    QVector3D position;
    QVector3D view;
    QVector3D up;
    QVector3D side;

    qreal scale;
    qreal nearPlane;
    qreal farPlane;
    qreal orthoA;
    qreal orthoB;
    int width;
    int height;

    void RotateAxes();
};

#endif // CAMERA_H
