/*
Example code for the technique presented in the paper

GPU-based Sphere Tracing for Radial Basis Functions Implicits
(Batagelo, H. C.; Gois, J. P.). International Journal of Image and Graphics,
14(1-2), 2014, DOI 10.1142/S0219467814500041

Copyright (c) 2014, Harlen Costa Batagelo
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * The name of the copyright holder may not be used to endorse or promote
      products derived from this software without specific prior written
      permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDER "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL HARLEN COSTA BATAGELO BE LIABLE FOR ANY
DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
(INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
(INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "mainwindow.h"
#include "ui_mainwindow.h"

MainWindow::MainWindow(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    ui->glWidget->isFullScreen = false;
    dlg = NULL;

    connect(ui->glWidget, SIGNAL(toggleFullScreen()),
            this, SLOT(toggleFullScreen()));
}

MainWindow::~MainWindow()
{
    delete ui;
}

void MainWindow::keyPressEvent(QKeyEvent *event)
{
    switch(event->key())
    {
    case Qt::Key_Enter:
    case Qt::Key_Return:
        if(event->modifiers() & Qt::AltModifier)
            toggleFullScreen();
        break;
    case Qt::Key_Escape:
        if (ui->glWidget->isFullScreen)
            toggleFullScreen();
        else
            qApp->quit();
        break;
    }
}

void MainWindow::toggleFullScreen()
{
    if (!ui->glWidget->isFullScreen) {
        if (dlg)
            delete dlg;
        dlg = new QDialog(this);
        QHBoxLayout *dlgLayout = new QHBoxLayout(dlg);
        dlgLayout->setContentsMargins(0, 0, 0, 0);
        dlgLayout->addWidget(ui->glWidget);
        dlg->setLayout(dlgLayout);
        dlg->showFullScreen();
        ui->glWidget->isFullScreen = true;
        ui->centralWidget->layout()->removeWidget(ui->optionsWidget);
    }
    else {
        dlg->accept();
        ui->centralWidget->layout()->addWidget(ui->glWidget);
        ui->centralWidget->layout()->addWidget(ui->optionsWidget);
        ui->glWidget->setFocus();
        ui->glWidget->isFullScreen = false;
    }
}


